@@ -1,5 +1,5 @@
+"_PACKAGE"
 #' @title Automating Mplus Model Estimation and Interpretation
-#' @docType package
 #' @name MplusAutomation
 #'
 #' @description The MplusAutomation package leverages the flexibility of the
@@ -10,12 +10,10 @@
 #'   and tabulating model parameters and fit statistics.
 #'
 #' @details The MplusAutomation package has four primary purposes:
-#' \enumerate{
-#'   \item To automatically run groups/batches of models.
-#'   \item To provide routines to extract model fit statistics, parameter estimates, and raw data from 'Mplus' output files.
-#'   \item To facilitate comparisons among models
-#'   \item To provide a template language that allows for the creation of related input files.
-#' }
+#' * To automatically run groups/batches of models.
+#' * To provide routines to extract model fit statistics, parameter estimates, and raw data from 'Mplus' output files.
+#' * To facilitate comparisons among models
+#' * To provide a template language that allows for the creation of related input files.
 #' The core routine for running batches of models is \code{\link{runModels}}, with
 #'   an easy-to-use GUI wrapper, \code{\link{runModels_Interactive}}.
 #'
@@ -39,12 +37,6 @@
 #'   or \code{\link{LatexSummaryTable}} (returns a LaTex-formatted table of
 #'   summary statistics).
 #'
-#' Deprecated: To extract raw data created by the SAVEDATA command (e.g., class membership probabilities
-#'   or factor scores), see \code{\link{getSavedata_Data}}.
-#'
-#' Deprecated: To extract unstandardized or standardized parameter estimates from a single output file,
-#'   see \code{\link{extractModelParameters}}.
-#'
 #' Detailed model fit and parameter comparisons between two models can be obtained using
 #'   \code{\link{compareModels}}.
 #'
@@ -59,8 +51,8 @@
 #' \tabular{ll}{
 #' Package: \tab MplusAutomation\cr
 #' Type: \tab Package\cr
-#' Version: \tab 1.1.1\cr
-#' Date: \tab 2024-01-27\cr
+#' Version: \tab 1.2\cr
+#' Date: \tab 2025-08-25\cr
 #' License: \tab LGPL-3\cr
 #' LazyLoad: \tab yes\cr
 #' }
@@ -78,6 +70,9 @@
 NULL
 
 
+# Handle NSE data.table usage inside submitModels functions, private in mplusModel R6
+if (getRversion() >= "2.15.1") utils::globalVariables(c("cnum", "V1", "sched", "private"))
+
 # display version number and citation when the package is loaded
 .onAttach <- function(libname, pkgname) {
 
