@@ -12,7 +12,7 @@
 extractParameters_1chunk <- function(filename, thisChunk, columnNames, sectionName) {
   if (isEmpty(thisChunk)) stop("Missing chunk to parse.\n  ", filename)
   if (isEmpty(columnNames)) stop("Missing column names for chunk.\n  ", filename)
-  if (missing(sectionName)) { sectionName <- "" } #right now, just use sectionName for R-SQUARE section, where there are no subheaders per se
+  if (missing(sectionName)) sectionName <- "" #right now, just use sectionName for R-SQUARE section, where there are no subheaders per se
   
   #R-SQUARE sections are not divided into the usual subheader sections, and the order of top-level headers is not comparable to typical output.
   #Create a single match for the whole section
@@ -39,73 +39,79 @@ extractParameters_1chunk <- function(filename, thisChunk, columnNames, sectionNa
               identical(x, c("Variable", "Estimate", "Variance")) ||
               identical(x, c("Posterior", "One-Tailed", "95%", "C.I.")) ||
               identical(x, c("Variable", "Estimate", "S.D.", "P-Value", "Lower", "2.5%", "Upper", "2.5%")) ||
-              identical(x, c("Observed", "Two-Tailed", "Rate", "of")) #r2 output for MI v8 with rate of missing
+              identical(x, c("Observed", "Two-Tailed", "Rate", "of")) || #r2 output for MI v8 with rate of missing
+              identical(x, c("Latent", "Two-Tailed", "Rate", "of")) ||
+              identical(x, c("Variable", "Estimate", "S.E.", "Est./S.E.", "P-Value", "Missing"))
           ) { TRUE } else { FALSE }
         })
 
     thisChunk <- thisChunk[!headerRows]
       
-    convertMatches <- data.frame(startline=1, keyword="R-SQUARE", varname=NA_character_, operator=NA_character_, endline=length(thisChunk))
+    convertMatches <- data.frame(element=1, keyword="R-SQUARE", varname=NA_character_, operator=NA_character_, endline=length(thisChunk))
   } else if (sectionName=="probability.scale") {
     #Probability scale output tends to fall in one uniform chunk. If it has header (e.g., Latent Class 1), these are handled upstream before parsing into chunks processed here.
-    convertMatches <- data.frame(startline=1, keyword="Probability.Scale", varname=NA_character_, operator=NA_character_, endline=length(thisChunk))
+    convertMatches <- data.frame(element=1, keyword="Probability.Scale", varname=NA_character_, operator=NA_character_, endline=length(thisChunk))
   } else {
-    #okay to match beginning and end of line because strip.white used in scan
-    matches <- gregexpr("^\\s*((Means|Thresholds|Intercepts|Variances|Item Difficulties|Item Locations|Item Categories|Item Guessing|Upper Asymptote|Residual Variances|Base Hazard Parameters|New/Additional Parameters|Scales|Dispersion|Steps)|([\\w_\\d+\\.#\\&\\^]+\\s+(BY|WITH|ON|\\|))|([\\w_\\d+\\.#\\&\\^]+\\s*\\|\\s*[\\w_\\d+\\.#\\&\\^]+\\s*(BY|WITH|ON)))\\s*$", thisChunk, perl=TRUE)
-
-    #more readable (than above) using ldply from plyr
-    convertMatches <- ldply(matches, function(row) data.frame(start=row, end=row+attr(row, "match.length")-1))
+    # okay to match beginning and end of line because strip.white used in scan
+    # Mar2025: switch to optimized friendlyGregexpr for speed
+    convertMatches <- friendlyGregexpr("^\\s*((Means|Thresholds|Intercepts|Variances|Item Difficulties|Item Locations|Item Categories|Item Guessing|Upper Asymptote|Residual Variances|Base Hazard Parameters|New/Additional Parameters|Scales|Dispersion|Steps)|([\\w_\\d+\\.#\\&\\^]+\\s+(BY|WITH|ON|\\|))|([\\w_\\d+\\.#\\&\\^]+\\s*\\|\\s*[\\w_\\d+\\.#\\&\\^]+\\s*(BY|WITH|ON)))\\s*$", thisChunk, perl=TRUE)
     
-    #beware faulty logic below... assumes only one match per line (okay here)
-    convertMatches$startline <- 1:nrow(convertMatches)
+    # sometimes chunks have no parameters because they are empty. e.g., stdyx for ex7.30
+    # in this case, return null
+    if (is.null(convertMatches)) return(NULL)
+    # if (nrow(convertMatches)==0L) return(NULL)
     
-    #only keep lines with a single match
-    #this removes rows that are -1 from gregexpr
-    convertMatches <- convertMatches[which(convertMatches$start > 0),]
-    #convertMatches <- subset(convertMatches, start > 0) #removed for compliance with CRAN
+    # Mar2025: for speed, switch to base R regex and lapply, avoiding repeated calls to data.frame
+    #develop a data.frame that divides into keyword matches versus variable matches
+    results <- lapply(seq_len(nrow(convertMatches)), function(i) {
+      row <- convertMatches[i, ]
+      match <- row$tag
+      
+      # no variable names or operators for elements such as means
+      if (match %in% c("Means", "Thresholds", "Intercepts", "Variances", "Residual Variances", "Base Hazard Parameters", 
+                       "New/Additional Parameters", "Scales", "Item Difficulties", "Item Locations", "Item Categories", 
+                       "Item Guessing", "Upper Asymptote", "Dispersion", "Steps")) {
+        return(c(keyword = make.names(match), slope = NA_character_, varname = NA_character_, operator = NA_character_))
+      } 
+      
+      # for regmatches, the first element is always the entire match, and subsequent elements are capturing groups
+      # paramHeader operator, like POS_AFFECT BY
+      param_match <- regmatches(match, regexec("^\\s*([\\w_\\d+\\.#\\&\\^]+)\\s+(BY|WITH|ON|\\|)\\s*$", match, perl=TRUE))[[1]]
+      if (length(param_match) > 1L) {
+        return(c(keyword = NA_character_, slope = NA_character_, varname = param_match[2], operator = param_match[3]))
+      }
+      
+      rslope_match <- regmatches(match, regexec("^\\s*([\\w_\\d+\\.#\\&\\^]+)\\s*\\|\\s*([\\w_\\d+\\.#\\&\\^]+)\\s*(BY|WITH|ON)\\s*$", match, perl=TRUE))[[1]]
+      if (length(rslope_match) > 1L) {
+        return(c(keyword = NA_character_, slope = rslope_match[2], varname = rslope_match[3], operator = rslope_match[4]))
+      }
+      
+      stop("failure to match keyword: ", match)
+    })
     
-    #sometimes chunks have no parameters because they are empty. e.g., stdyx for ex7.30
-    #in this case, return null
-    if (nrow(convertMatches)==0) { return(NULL) }
-    
-    #develop a dataframe that divides into keyword matches versus variable matches
-    convertMatches <- ddply(convertMatches, "startline", function(row) {
-          #pull the matching keyword based on the start/end attributes from gregexpr
-          match <- substr(thisChunk[row$startline], row$start, row$end)
-
-          #check for keyword
-          if (match %in% c("Means", "Thresholds", "Intercepts", "Variances", "Residual Variances", "Base Hazard Parameters", "New/Additional Parameters", 
-              "Scales", "Item Difficulties", "Item Locations", "Item Categories", "Item Guessing", "Upper Asymptote", "Dispersion", "Steps")) {
-            return(data.frame(startline=row$startline, keyword=make.names(match), varname=NA_character_, operator=NA_character_, stringsAsFactors=FALSE))
-          } else if (length(variable <- strapply(match, "^\\s*([\\w_\\d+\\.#\\&\\^]+)\\s+(BY|WITH|ON|\\|)\\s*$", c, perl=TRUE)[[1]]) > 0) { #typical "factor BY" heading  
-            return(data.frame(startline=row$startline, keyword=NA_character_, varname=variable[1], operator=variable[2], stringsAsFactors=FALSE))
-          } else if (length(variable <- strapply(match, "^\\s*([\\w_\\d+\\.#\\&\\^]+)\\s*\\|\\s*([\\w_\\d+\\.#\\&\\^]+)\\s*(BY|WITH|ON)\\s*$", c, perl=TRUE)[[1]]) > 0) { #random slope syntax
-            return(data.frame(startline=row$startline, keyword=NA_character_, slope=variable[1], varname=variable[2], operator=variable[3], stringsAsFactors=FALSE))
-          } else stop("failure to match keyword: ", match, "\n  ", filename)
-        })
+    char_mat <- do.call(rbind, results)
+    convertMatches <- data.frame(element=convertMatches$element, char_mat) # convert to data.frame and restore element
   }
-  
-  comboFrame <- c()
 
   #convertMatches will now contain a data.frame marking the section headers for the chunk
   #example:
-  #		startline keyword varname operator endline
+  #		element keyword varname operator endline
   #		        7    <NA>      FW       BY      12
   #		       13    <NA>      FW       ON      16
-
-  for (i in 1:nrow(convertMatches)) {
+  
+  combo_list <- lapply(seq_len(nrow(convertMatches)), function(i) {
     #define the end line for this match as the start of next match - 1
-    if (i < nrow(convertMatches)) convertMatches[i,"endline"] <- convertMatches[i+1,"startline"]-1
+    if (i < nrow(convertMatches)) convertMatches[i,"endline"] <- convertMatches[i+1,"element"]-1L
     else convertMatches[i,"endline"] <- length(thisChunk) # or if last chunk in the section, just define as length
 
     #need +1 to eliminate header row from params
-    paramsToParse <- thisChunk[(convertMatches[i, "startline"]+1):convertMatches[i, "endline"]]
+    paramsToParse <- thisChunk[(convertMatches[i, "element"]+1L):convertMatches[i, "endline"]]
 
     #Very rarely, a section header is printed, but no parameters follow. In such cases, skip to the next match
     #example:
     #  I1       |
     #
-    if (all(paramsToParse=="")) { next }
+    if (all(paramsToParse=="")) return(NULL) # next
     
     #should result in a short list of params to parse (that belong to a given header i)
     #Example:
@@ -139,7 +145,7 @@ extractParameters_1chunk <- function(filename, thisChunk, columnNames, sectionNa
     if (columnNames[2L] == "est") {
       splitParams <- lapply(splitParams, function(x) {
         if (length(x) >= 2L && grepl("undefined", x[2L], ignore.case = TRUE)) { 
-          x <- c(x[1L], rep("NA_real_", length(columnNames) - 1)) #Fill in NAs for undefined params
+          x <- c(x[1L], rep("NA", length(columnNames) - 1)) #Fill in NAs for undefined params
         }
         return(x)
       })
@@ -162,7 +168,7 @@ extractParameters_1chunk <- function(filename, thisChunk, columnNames, sectionNa
     if (length(columnNames) == 6L && (columnNames[6L] == "scale_f" || columnNames[6L] == "resid_var")) {
       splitParams <- lapply(splitParams, function(col) {
         lcol <- length(col)
-        if (lcol == 5L) { col[6L] <- "NA_real_" } #NA-fill variables without a scale factor or residual variance
+        if (lcol == 5L) { col[6L] <- "NA" } #NA-fill variables without a scale factor or residual variance
         return(col)
       })
     }
@@ -171,7 +177,7 @@ extractParameters_1chunk <- function(filename, thisChunk, columnNames, sectionNa
     if (length(columnNames) == 3L && columnNames[3L] %in% c("resid_var", "scale_f")) {
       splitParams <- lapply(splitParams, function(col) {
         lcol <- length(col)
-        if (lcol == 2L) { col[3L] <- "NA_real_" } #NA-fill variables without a residual variance
+        if (lcol == 2L) { col[3L] <- "NA" } #NA-fill variables without a residual variance
         return(col)
       })
     }
@@ -179,17 +185,10 @@ extractParameters_1chunk <- function(filename, thisChunk, columnNames, sectionNa
     #rbind the split list as a data.frame
     parsedParams <- data.frame(do.call("rbind", splitParams), stringsAsFactors=FALSE)
 
-    #for each column, convert to numeric if it is. Otherwise, return as character
-    parsedParams <- data.frame(lapply(parsedParams, function(col) {
-      #a bit convoluted, but we want to test for a purely numeric string by using a regexp that only allows numbers, periods, and the minus sign
-      #then sum the number of matches > 0 (i.e., where a number was found).
-      #if the sum is the same as the length of the column, then all elements are purely numeric.
-      if (all(col %in% c("TRUE", "FALSE"))) return(as.logical(col)) #True/False significance check above
-      else if (sum(sapply(gregexpr("^(NA_real_|[\\d\\.-]+)$", col, perl=TRUE), "[", 1) > 0) == length(col)) return(suppressWarnings(as.numeric(col)))
-      else return(as.character(col))
-    }), stringsAsFactors=FALSE)
-
-
+    # At present, parsedParams is a data.frame of all character variables.
+    # For each column, convert to numeric, logical, or character if it is. Otherwise, return as character
+    parsedParams[] <- lapply(parsedParams, type.convert, as.is=TRUE)  # Apply transformation to each column
+    
     #use the column names detected in extractParameters_1section
     names(parsedParams) <- columnNames
 
@@ -198,11 +197,12 @@ extractParameters_1chunk <- function(filename, thisChunk, columnNames, sectionNa
 
     #put the paramHeader at the front of the data.frame columns
     parsedParams <- parsedParams[,c("paramHeader", columnNames)]
-
-    #add the current chunk to the overall data.frame
-    comboFrame <- rbind(comboFrame, parsedParams)
-
-  }
+    
+    return(parsedParams)
+  })
+  
+  #combined the chunks into one data.frame
+  comboFrame <- do.call(rbind, combo_list)
 
   if (sectionName == "r2") { comboFrame$paramHeader <- NULL } #not relevant for this section
   
@@ -240,7 +240,7 @@ extractParameters_1section <- function(filename, modelSection, sectionName) {
 
   #detectColumnNames sub-divides (perhaps unnecessarily) the matches based on the putative section type of the output
   #current distinctions include modification indices, confidence intervals, and model results.
-  if (sectionName %in% c("ci.unstandardized", "ci.stdyx.standardized", "ci.stdy.standardized", "ci.std.standardized")) { 
+  if (sectionName %in% c("ci.unstandardized", "ci.stdyx.standardized", "ci.stdy.standardized", "ci.std.standardized", "ci.odds")) { 
     sectionType <- "confidence_intervals"
   } else if (sectionName == "irt.parameterization" || sectionName == "probability.scale") {
     #the IRT section follows from the MODEL RESULTS section, and column headers are not reprinted.
@@ -251,8 +251,15 @@ extractParameters_1section <- function(filename, modelSection, sectionName) {
     columnNames <- suppressWarnings(detectColumnNames(filename, modelSection, "model_results"))
     
     #If we can't detect column names (because they're not reprinted from model results, assume a 5-column header -- heuristic
-    if (is.null(columnNames)) { columnNames <- c("param", "est", "se", "est_se", "pval") } #default if detection fails
-  } else { sectionType <- "model_results" }
+    if (is.null(columnNames)) columnNames <- c("param", "est", "se", "est_se", "pval") #default if detection fails
+  } else if (sectionName == "odds") {
+    columnNames <- suppressWarnings(detectColumnNames(filename, modelSection, "model_results"))
+    
+    #If we can't detect column names (because they're not printed), assume that it's just param and est. Happens in Mplus v8, for example.
+    if (is.null(columnNames)) columnNames <- c("param", "est") #default if detection fails
+  } else { 
+    sectionType <- "model_results" 
+  }
 
   if (!exists("columnNames")) { columnNames <- detectColumnNames(filename, modelSection, sectionType) }
 
@@ -278,20 +285,23 @@ extractParameters_1section <- function(filename, modelSection, sectionName) {
   catLatentMatches <- grep("^\\s*Categorical Latent Variables\\s*$", modelSection, ignore.case=TRUE)
   classPropMatches <- grep("^\\s*Class Proportions\\s*$", modelSection, ignore.case=TRUE)
   classSpecificMatches <- grep("^\\s*(Results|Parameters) for Class-specific Model Parts of [\\w_\\.]+\\s*$", modelSection, ignore.case=TRUE, perl=TRUE)
+  referenceClassMatches <- grep("^\\s*Parameterization using Reference Class [0-9]+", modelSection, ignore.case=TRUE)
 
-  topLevelMatches <- sort(c(betweenWithinMatches, latentClassMatches, multipleGroupMatches, catLatentMatches, classPropMatches, classSpecificMatches))
+  topLevelMatches <- sort(c(betweenWithinMatches, latentClassMatches, multipleGroupMatches, catLatentMatches, classPropMatches, classSpecificMatches, referenceClassMatches))
   
   if (length(topLevelMatches) > 0) {
 
     lcNum <- NULL
     bwWi <- NULL
     groupName <- NULL
+    rcNum <- NULL
 
     matchIndex <- 1
     for (match in topLevelMatches) {
       if (grepl("\\s*Within-Level Standardized Estimates Averaged Over Clusters\\s*", modelSection[match], perl=TRUE)) {
         bwWi <- "Within.Std.Averaged.Over.Clusters"
-      } else if (match %in% betweenWithinMatches) { bwWi <- sub("\\s+Level\\s*$", "", modelSection[match], perl=TRUE)
+      } else if (match %in% betweenWithinMatches) { 
+        bwWi <- sub("\\s+Level\\s*$", "", modelSection[match], perl=TRUE)
       } else if (match %in% latentClassMatches) {
         if ((pos <- regexpr("Pattern", modelSection[match], ignore.case=TRUE)) > 0) {
           #need to pull out and concatenate all numerical values following pattern
@@ -320,6 +330,10 @@ extractParameters_1section <- function(filename, modelSection, sectionName) {
         #N.B.: 15Mar2012. the parse chunk routine can't handle the class proportions output right now
         #because there is no nesting. Need to come back and fix.
         #for now, this output is just ignored.
+      } else if (match %in% referenceClassMatches) {
+        lcNum <- NULL
+        bwWi <- NULL
+        rcNum <- sub("^\\s*Parameterization using Reference Class ([0-9]+)\\s*$", "\\1", modelSection[match], perl=TRUE)
       }
 
       #if the subsequent top level match is more than 2 lines away, assume that there is a
@@ -395,6 +409,7 @@ extractParameters_1section <- function(filename, modelSection, sectionName) {
           parsedChunk$LatentClass <- lcNum
           parsedChunk$BetweenWithin <- bwWi
           parsedChunk$Group <- groupName
+          parsedChunk$ReferenceClass <- rcNum
           allSectionParameters <- rbind(allSectionParameters, parsedChunk)
         }
       }
@@ -464,20 +479,6 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
   #   warning(paste0("EFA, MIXTURE EFA, and TWOLEVEL EFA files are not currently supported by extractModelParameters.\n  Skipping outfile: ", filename))
   #   return(NULL) #skip file
   # }
-
-  # copy elements of append into target. note that data.frames inherit list,
-  # so could be wonky if append is a data.frame (shouldn't happen here)
-  appendListElements <- function(target, append) {
-    if (!is.list(target)) stop("target is not a list.")
-    if (!is.list(append)) stop("append is not a list.")
-
-    for (elementName in names(append)) {
-      if (!is.null(target[[elementName]])) warning("Element is already present in target list: ", elementName)
-      target[[elementName]] <- append[[elementName]]
-    }
-
-    return(target)
-  }
   
   allSections <- list() #holds parameters for all identified sections
   if (length(multisectionMatches <- grep("^\\s*MODEL RESULTS FOR .*", outfiletext, perl=TRUE, value=TRUE)) > 0L) {
@@ -488,7 +489,7 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
       unstandardizedSection <- getSection(multisectionMatches[s], outfiletext)
       if (!is.null(unstandardizedSection)) {
         unstandardizedList[[ sectionNames[s] ]] <- extractParameters_1section(filename, unstandardizedSection, "unstandardized")[[1]]
-      }      
+      }
     }
     allSections$unstandardized <- unstandardizedList
   } else {
@@ -498,16 +499,16 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
       allSections <- appendListElements(allSections, extractParameters_1section(filename, unstandardizedSection, "unstandardized"))
     }
     
-    alt.undstandardized <- getSection("^ALTERNATIVE PARAMETERIZATIONS FOR THE CATEGORICAL LATENT VARIABLE REGRESSION$", outfiletext)
+    alt.unstandardized <- getSection("^ALTERNATIVE PARAMETERIZATIONS FOR THE CATEGORICAL LATENT VARIABLE REGRESSION$", outfiletext)
     
-    if (!is.null(alt.undstandardized)) {
-      header.idx <- c(grep("Parameterization using Reference Class (\\d+)", alt.undstandardized), length(alt.undstandardized))
+    if (!is.null(alt.unstandardized)) {
+      header.idx <- c(grep("Parameterization using Reference Class (\\d+)", alt.unstandardized), length(alt.unstandardized))
       
       alt.params <- llply(1:(length(header.idx)-1), function(i) {
         start.line <- header.idx[[i]]
         end.line <- header.idx[[i+1]]-1
         
-        alt.section <- c(alt.undstandardized[1:4], alt.undstandardized[start.line:end.line])
+        alt.section <- c(alt.unstandardized[1:4], alt.unstandardized[start.line:end.line])
         
         res <- extractParameters_1section(filename, alt.section, "alt")
         
@@ -624,10 +625,18 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
     probParsed[[1]] <- probParsed[[1]][,c("param", "category", "est", other_cols)] #reorder columns
     allSections <- appendListElements(allSections, probParsed)
   }
+  
+  # odds ratios
+  oddsSection <- getSection("^LOGISTIC REGRESSION ODDS RATIO RESULTS$", outfiletext)
+  if (!is.null(oddsSection)) {
+    allSections <- appendListElements(allSections, extractParameters_1section(filename, oddsSection, "odds"))
+  }
 
   #confidence intervals for usual output, credibility intervals for bayesian output
   ciSection <- getSection("^(CONFIDENCE INTERVALS OF MODEL RESULTS|CREDIBILITY INTERVALS OF MODEL RESULTS)$", outfiletext)
   if (!is.null(ciSection)) {
+    # capture the header line to be reused by the odds section, if relevant (where the header is not reprinted)
+    hline <- detectColumnNames(filename, ciSection, "confidence_intervals")
     allSections <- appendListElements(allSections, extractParameters_1section(filename, ciSection, "ci.unstandardized"))
   }
 
@@ -643,6 +652,13 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
     std.section <- getSection("STD Standardization", ciStdSection, headers=stdsections)
     if (!is.null(std.section)) allSections <- appendListElements(allSections, extractParameters_1section(filename, std.section, "ci.std.standardized"))
   }
+  
+  # when the confidence intervals for logistic regression odds are present, the header line is not reprinted and must be copied from the 
+  ciOddsSection <- getSection("^CONFIDENCE INTERVALS FOR THE LOGISTIC REGRESSION ODDS RATIO RESULTS$", outfiletext)
+  if (!is.null(ciOddsSection)) {
+    ciOddsSection <- c(ciSection[attr(hline, "header_lines")], ciOddsSection)
+    allSections <- appendListElements(allSections, extractParameters_1section(filename, ciOddsSection, "ci.odds")) 
+  }
 
   #extract EFA parameters if this is an EFA output
   if (efa) {
@@ -655,7 +671,7 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
 
   # cleaner equivalent of above
   listOrder <- c("unstandardized", "unstandardized.alt", "r2", "ci.unstandardized",
-      "irt.parameterization", "probability.scale",
+      "irt.parameterization", "probability.scale", "odds", "ci.odds",
       "stdyx.standardized", "ci.stdyx.standardized",
       "stdy.standardized", "ci.stdy.standardized",
       "std.standardized", "ci.std.standardized",
@@ -686,7 +702,7 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
   return(allSections)
 }
 
-#' Extract model parameters from MODEL RESULTS section.
+#' (DEPRECATED) Extract model parameters from MODEL RESULTS section.
 #'
 #' Extracts the model parameters from the MODEL RESULTS section of one or more Mplus output files.
 #' If a particular output file has more than one results section (unstandardized, stdyx, stdy, and/or std),
@@ -700,7 +716,7 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
 #' @param recursive optional. If \code{TRUE}, parse all models nested in subdirectories
 #'   within \code{target}. Defaults to \code{FALSE}.
 #' @param filefilter a Perl regular expression (PCRE-compatible) specifying particular output
-#'   files to be parsed within \code{directory}. See \code{regex} or \url{http://www.pcre.org/pcre.txt}
+#'   files to be parsed within \code{directory}. See \code{regex} or \url{https://www.pcre.org/pcre.txt}
 #'   for details about regular expression syntax.
 #' @param dropDimensions Relevant only for multi-file parsing. If \code{TRUE}, then if only one output
 #'   section (usually unstandardized) is present for all files in the parsed list, then eliminate
@@ -725,40 +741,40 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
 #'
 #' The core \code{data.frame} for each MODEL RESULTS section typically has the following structure:
 #'
-#' \item{paramHeader}{The header that begins a given parameter set. Example: "FACTOR1 BY"}
-#' \item{param}{The particular parameter being measured (within \code{paramHeader}). Example: "ITEM1"}
-#' \item{est}{Parameter estimate value.}
-#' \item{se}{Standard error of the estimate}
-#' \item{est_se}{Quotient of \code{est/se}, representing z-test/t-test in large samples}
-#' \item{pval}{Two-tailed p-value for the \code{est_se} quotient.}
+#' * `paramHeader`: The header that begins a given parameter set. Example: "FACTOR1 BY"
+#' * `param`: The particular parameter being measured (within \code{paramHeader}). Example: "ITEM1"
+#' * `est`: Parameter estimate value.
+#' * `se`: Standard error of the estimate
+#' * `est_se`: Quotient of \code{est/se}, representing z-test/t-test in large samples
+#' * `pval`: Two-tailed p-value for the \code{est_se} quotient.
 #'
 #' In the case of output from Bayesian estimation (ESTIMATOR=BAYES), the \code{data.frame} will contain
 #' a different set of variables, including some of the above, as well as
-#' \item{posterior_sd}{Posterior standard deviation of the estimate.}
-#' \item{lower_2.5ci}{Lower 2.5 percentile of the estimate.}
-#' \item{upper_2.5ci}{Upper 2.5 percentile (aka 97.5 percentile) of the estimate.}
+#' * `posterior_sd`: Posterior standard deviation of the estimate.
+#' * `lower_2.5ci`: Lower 2.5 percentile of the estimate.
+#' * `upper_2.5ci`: Upper 2.5 percentile (aka 97.5 percentile) of the estimate.
 #'
 #' Also note that the \code{pval} column for Bayesian output represents a one-tailed estimate.
 #'
 #' In the case of output from a Monte Carlo study (MONTECARLO: and MODEL POPULATION:), the \code{data.frame} will contain
 #' a different set of variables, including some of the above, as well as
-#' \item{population}{Population parameter value.}
-#' \item{average}{Average parameter estimate across replications.}
-#' \item{population_sd}{Standard deviation of parameter value in population across replications.}
-#' \item{average_se}{Average standard error of estimated parameter value across replications.}
-#' \item{mse}{Mean squared error.}
-#' \item{cover_95}{Proportion of replications whose 95\% confidence interval for the parameter includes the population value.}
-#' \item{pct_sig_coef}{Proportion of replications for which the two-tailed significance test of the parameter is significant (p < .05).}
+#' * `population`: Population parameter value.
+#' * `average`: Average parameter estimate across replications.
+#' * `population_sd`: Standard deviation of parameter value in population across replications.
+#' * `average_se`: Average standard error of estimated parameter value across replications.
+#' * `mse`: Mean squared error.
+#' * `cover_95`: Proportion of replications whose 95\% confidence interval for the parameter includes the population value.
+#' * `pct_sig_coef`: Proportion of replications for which the two-tailed significance test of the parameter is significant (p < .05).
 #'
 #' In the case of confidence interval output (OUTPUT:CINTERVAL), the list element \code{ci.unstandardized} will contain
 #' a different set of variables, including some of the above, as well as
-#' \item{low.5}{Lower 0.5\% CI estimate.}
-#' \item{low2.5}{Lower 2.5\% CI estimate.}
-#' \item{low5}{Lower 5\% CI estimate.}
-#' \item{est}{Parameter estimate value.}
-#' \item{up5}{Upper 5\% (i.e., 95\%) CI estimate.}
-#' \item{up2.5}{Upper 2.5\% (i.e., 97.5\%) CI estimate.}
-#' \item{up.5}{Upper 0.5\% (i.e., 99.5\%) CI estimate.}
+#' * `low.5`: Lower 0.5\% CI estimate.
+#' * `low2.5`: Lower 2.5\% CI estimate.
+#' * `low5`: Lower 5\% CI estimate.
+#' * `est`: Parameter estimate value.
+#' * `up5`: Upper 5\% (i.e., 95\%) CI estimate.
+#' * `up2.5`: Upper 2.5\% (i.e., 97.5\%) CI estimate.
+#' * `up.5`: Upper 0.5\% (i.e., 99.5\%) CI estimate.
 #'
 #' If the model contains multiple latent classes, an additional variable, \code{LatentClass},
 #' will be included, specifying the latent class number. Also, the Categorical Latent Variables section
@@ -771,8 +787,7 @@ extractParameters_1file <- function(outfiletext, filename, resultType, efa = FAL
 #'
 #' @author Michael Hallquist
 #' @seealso \code{\link{extractModelSummaries}}
-#' @export
-#' @keywords interface
+#' @keywords internal
 #' @examples
 #' \dontrun{
 #' ex3.14 <- extractModelParameters(
@@ -788,4 +803,4 @@ extractModelParameters <- function(target=getwd(), recursive=FALSE, filefilter,
   #extractParameters_1section: extract model parameters for a given section.
   #extractParameters_1chunk: extract model parameters for a given chunk (e.g., Latent class 2, Between Level) within a given section.
 
-}
\ No newline at end of file
+}
