@@ -186,44 +186,53 @@ testBParamCompoundConstraint <- function(bparams, test) {
 #' @param charvector Character vector
 #' @param perl A logical whether or not to use perl based
 #'   regular expressions.  Defaults to \code{TRUE}.
+#' @param include_tag if TRUE, return the match as a character string. This is the default, but
+#'   setting to `FALSE` is a bit faster
 #' @return A \code{data.frame}
 #' @author Michael Hallquist
 #' @keywords internal
 #' @examples
-#' ## make me
-friendlyGregexpr <- function(pattern, charvector, perl=TRUE) {
-  #now create data frame documenting the start and end of all tags
-  #rather than ldply, need a usual loop to track element number (in cases where charvector is a vector)
-  regexpMatches <- gregexpr(pattern, charvector, perl=perl)
+#' \dontrun{
+#'   friendlyGregexpr("(BY|WITH|ON)", 
+#'     c("POS_WI BY", "X WITH Y WITH Z")
+#'   )
+#' } 
+friendlyGregexpr <- function(pattern, charvector, perl = TRUE, include_tag = TRUE) {
+  regexpMatches <- gregexpr(pattern, charvector, perl = perl)
   
-  convertMatches <- c()
-  for (i in 1:length(regexpMatches)) {
+  matches_list <- vector("list", length(regexpMatches))
+  index <- 1
+  
+  for (i in seq_along(regexpMatches)) {
     thisLine <- regexpMatches[[i]]
-    #only append if there is at least one match on this line
     if (thisLine[1] != -1) {
-      convertMatches <- rbind(convertMatches, data.frame(element=i, start=thisLine, end=thisLine+attr(thisLine, "match.length")-1))
+      match.lengths <- attr(thisLine, "match.length")
+      start <- as.integer(thisLine) # remove attributes
+      
+      matches_list[[index]] <- matrix(data=c(
+        rep(i, length(thisLine)), start, start + match.lengths - 1L),
+        ncol=3, dimnames=list(NULL, c("element", "start", "end"))
+      )
+      
+      index <- index + 1
     }
   }
   
-  #if no matches exist, return null (otherwise, will break adply)
-  if (is.null(convertMatches)) return(NULL)
+  # Combine results if there are matches
+  convertMatches <- if (index > 1) as.data.frame(do.call(rbind, matches_list[1:(index - 1)])) else return(NULL)
   
-  #okay, now we have a data frame with the line, starting position, and ending position of every tag
+  # Add 'tag' column using vectorized substr()
+  if (isTRUE(include_tag)) {
+    convertMatches$tag <- mapply(
+      function(elem, start, end) substr(charvector[elem], start, end),
+      convertMatches$element, convertMatches$start, convertMatches$end
+    )
+  }
   
-  #time to classify into simple, array, iterator, and conditional
-  
-  #first, add the actual tag to the data.frame to make it easier to parse
-  #using adply (is this not its intended use?) to iterate over rows and apply func
-  convertMatches <- adply(convertMatches, 1, function(row) {
-        row$tag <- substr(charvector[row$element], row$start, row$end)
-        #for some reason, adply does not respect the stringsAsFactors here
-        return(as.data.frame(row, stringAsFactors=FALSE))
-      })
-  
-  convertMatches$tag <- as.character(convertMatches$tag)
   return(convertMatches)
 }
 
+
 #expose as root-level function to be used by model summary extraction
 #METHOD NOW DEPRECATED. USING KEYWORD-MATCHING APPROACH BELOW
 getSection_Blanklines <- function(sectionHeader, outfiletext) {
@@ -334,8 +343,8 @@ getSection <- function(sectionHeader, outfiletext, headers="standard") {
   #identify headers after this section to find end of section
   subsequentHeaders <- which(h > beginSection)
   
-  if (length(subsequentHeaders) == 0) { nextHeader <- length(outfiletext) #just return the whole enchilada
-  } else { nextHeader <- h[subsequentHeaders[1]] - 1 }
+  if (length(subsequentHeaders) == 0) nextHeader <- length(outfiletext) #just return the whole enchilada
+  else nextHeader <- h[subsequentHeaders[1]] - 1
   
   section.found <- outfiletext[(beginSection+1):nextHeader]
   attr(section.found, "lines") <- beginSection:nextHeader
@@ -358,6 +367,7 @@ parse_into_sections <- function(outfiletext) {
       "RESULTS FOR BASIC ANALYSIS",
       "CROSSTABS FOR CATEGORICAL VARIABLES", "UNIVARIATE PROPORTIONS AND COUNTS FOR CATEGORICAL VARIABLES",
       "SUMMARY OF CENSORED LIMITS", "COUNT PROPORTION OF ZERO, MINIMUM AND MAXIMUM VALUES",
+      "RANDOM STARTS RESULTS RANKED FROM THE BEST TO THE WORST FIT FUNCTION VALUES",
       "RANDOM STARTS RESULTS RANKED FROM THE BEST TO THE WORST LOGLIKELIHOOD VALUES",
       "TESTS OF MODEL FIT", "MODEL FIT INFORMATION", "MODEL FIT INFORMATION FOR .*", "CLASSIFICATION QUALITY",
       "SUMMARY OF MODEL FIT INFORMATION", "RESULTS FOR EXPLORATORY FACTOR ANALYSIS",
@@ -385,6 +395,7 @@ parse_into_sections <- function(outfiletext) {
       "BETWEEN-LEVEL FACTOR SCORE COMPARISONS",
       "ALTERNATIVE PARAMETERIZATIONS FOR THE CATEGORICAL LATENT VARIABLE REGRESSION",
       "ODDS RATIO FOR THE ALTERNATIVE PARAMETERIZATIONS FOR THE CATEGORICAL LATENT VARIABLE REGRESSION",
+      "ODDS RATIOS FOR TESTS OF CATEGORICAL LATENT VARIABLE MULTINOMIAL LOGISTIC REGRESSIONS",
       "LATENT CLASS ODDS RATIO RESULTS", "LOGRANK OUTPUT", "STANDARDIZED MODEL RESULTS",
       "WITHIN-LEVEL STANDARDIZED MODEL RESULTS FOR CLUSTER \\d+",
       "R-SQUARE", "QUALITY OF NUMERICAL RESULTS", "QUALITY OF NUMERICAL RESULTS FOR .*", "TECHNICAL OUTPUT", "TECHNICAL \\d+ OUTPUT",
@@ -401,15 +412,19 @@ parse_into_sections <- function(outfiletext) {
       "CONFIDENCE INTERVALS OF TOTAL, TOTAL INDIRECT, SPECIFIC INDIRECT, AND DIRECT EFFECTS",
       "CONFIDENCE INTERVALS OF STANDARDIZED TOTAL, TOTAL INDIRECT, SPECIFIC INDIRECT,", #omitted "AND DIRECT EFFECTS" in v7
       "CONFIDENCE INTERVALS OF STANDARDIZED TOTAL, TOTAL INDIRECT, SPECIFIC INDIRECT, AND DIRECT EFFECTS", #fit onto 1 line in v8!
+      "CONFIDENCE INTERVALS FOR TESTS OF CATEGORICAL LATENT VARIABLE MULTINOMIAL LOGISTIC REGRESSIONS", # 3-step output in v8.10
+      "CONFIDENCE INTERVALS OF ODDS RATIOS FOR TESTS OF CATEGORICAL LATENT VARIABLE MULTINOMIAL", # 3-step output in v8.10
       "EQUALITY TESTS OF MEANS ACROSS CLASSES USING POSTERIOR PROBABILITY-BASED",
       "EQUALITY TESTS OF MEANS ACROSS CLASSES USING THE BCH PROCEDURE",
       "EQUALITY TESTS OF MEANS ACROSS CLASSES USING THE 3-STEP PROCEDURE",
       "EQUALITY TESTS OF MEANS/PROBABILITIES ACROSS CLASSES",
+      "TESTS OF CATEGORICAL LATENT VARIABLE MULTINOMIAL LOGISTIC REGRESSIONS USING", # 3-step regression
+      "DIFFERENCE OUTPUT", # differences in priors
       "THE FOLLOWING DATA SET\\(S\\) DID NOT RESULT IN A COMPLETED REPLICATION:",
       "RESIDUAL OUTPUT", "RESIDUAL OUTPUT FOR THE.*", 
       "MODEL MODIFICATION INDICES", "MODIFICATION INDICES", 
       "MODEL COMMAND WITH FINAL ESTIMATES USED AS STARTING VALUES",
-      "SUMMARIES OF PLAUSIBLE VALUES \\(N = NUMBER OF OBSERVATIONS * NUMBER OF IMPUTATIONS\\)",
+      "SUMMARIES OF PLAUSIBLE VALUES \\(N = NUMBER OF OBSERVATIONS \\* NUMBER OF IMPUTATIONS\\)",
       "SUMMARY OF PLAUSIBLE STANDARD DEVIATION \\(N = NUMBER OF OBSERVATIONS\\)",
       "Available post-processing tools:",
       "FACTOR SCORE INFORMATION \\(COMPLETE DATA\\)", "SUMMARY OF FACTOR SCORES", "PLOT INFORMATION", "SAVEDATA INFORMATION",
@@ -635,13 +650,14 @@ parseCatOutput <- function(text) {
 #' @param target The target file or directory
 #' @param recursive A logical value whether to search recursively.
 #'   Defaults to \code{FALSE}.
-#' @param filefilter A regular expression passed to \code{grep}
-#'   used to filter the output files.
+#' @param filefilter A PCRE regular expression passed to \code{grep}
+#'   used to filter the output files based on their file names
+#' @param pathfilter A PCRE regular expression passed to \code{grep}
+#'   used to filter the output files based on their absolute paths
 #' @return A character vector of the output files
 #' @keywords internal
-#' @examples
-#' # make me!!!
-getOutFileList <- function(target, recursive=FALSE, filefilter) {
+#' @importFrom checkmate assert_string
+getOutFileList <- function(target, recursive=FALSE, filefilter, pathfilter) {
 #could this also be used by runModels to locate input files?
 #seems like that function would do well to allow for directories and single files, too.
   
@@ -649,18 +665,63 @@ getOutFileList <- function(target, recursive=FALSE, filefilter) {
   if (file.exists(target)) {
     if (isTRUE(file.info(target)$isdir)) {
       
+      # always convert to absolute path for filtering
+      target <- normalizePath(target)
+      
       #obtain list of all files in the specified directory
-      filelist <- list.files(path=target, recursive=recursive, full.names=TRUE)
+      filelist <- list.files(path=target, recursive=recursive, full.names=FALSE)
       
       #retain only .out files
       outfiles <- filelist[grep(".*\\.out$", filelist, ignore.case=TRUE)]
       
-      if (!missing(filefilter)) {
-        dropOutExtensions <- sapply(outfiles, function(x) {
-              if (nchar(x) >= 4) return(tolower(substr(x, 1, (nchar(x)-4))))
-            })
-        outfiles <- outfiles[grep(paste(".*", filefilter, ".*", sep=""), dropOutExtensions, ignore.case=TRUE, perl=TRUE)]
+      # for filtering paths
+      paths <- normalizePath(file.path(target, dirname(outfiles)))
+      
+      # ensure that outfiles only contains filenames, no directories
+      outfiles <- basename(outfiles)
+      
+      if (missing(filefilter)) {
+        good_files <- seq_along(outfiles)
+      } else {
+        checkmate::assert_string(filefilter)
+        # allow filter to include .out, and to specify start (^) and end ($) characters
+        firstChar <- ifelse(substr(filefilter, 1, 1) == "^", "^", ".*")
+        lastChar <- ".*"
+        dropOut <- TRUE
+        if (substr(filefilter, nchar(filefilter) - 4, nchar(filefilter)) == "\\.out") {
+          lastChar <- "$" # enforce EOL after .out
+          dropOut <- FALSE
+        } else if (substr(filefilter, nchar(filefilter) - 5, nchar(filefilter)) == "\\.out$") {
+          lastChar <- "" # EOL already present
+          dropOut <- FALSE
+        }
+        
+        if (dropOut) {
+          dropOutExtensions <- sapply(outfiles, function(x) {
+            if (nchar(x) >= 4) return(tolower(substr(x, 1, (nchar(x)-4))))
+          })
+          good_files <- grep(paste(firstChar, filefilter, lastChar, sep=""), dropOutExtensions, ignore.case=TRUE, perl=TRUE)
+        } else {
+          good_files <- grep(paste(firstChar, filefilter, lastChar, sep=""), outfiles, ignore.case=TRUE, perl=TRUE)
+        }
       }
+      
+      if (missing(pathfilter)) {
+        good_paths <- seq_along(outfiles)
+      } else {
+        checkmate::assert_string(pathfilter)
+        
+        # allow filter to include .out, and to specify start (^) and end ($) characters
+        firstChar <- ".*"
+        if (substr(pathfilter, 1, 1) == "^") firstChar <- "^"
+        lastChar <- ".*"
+        if (substr(pathfilter, nchar(pathfilter), nchar(pathfilter)) == "$") lastChar <- "" # EOL already present
+
+        good_paths <- grep(paste(firstChar, pathfilter, lastChar, sep=""), paths, ignore.case=TRUE, perl=TRUE)
+      }
+      
+      keep <- intersect(good_paths, good_files)
+      outfiles <- file.path(paths[keep], outfiles[keep])
     }
     else {
       #ensure that target is a single output file.
@@ -685,20 +746,22 @@ getOutFileList <- function(target, recursive=FALSE, filefilter) {
 #' This is a helper function to split path into path and filename.
 #' Code adapted from R.utils filePath command.
 #'
-#' @param abspath A character string of the file path
+#' @param filepath A character string of the file path
 #' @return A list with elements for the directory, filename,
 #'   and absolute path.
 #' @keywords internal
 #' @examples
-#' # make me!!!
-splitFilePath <- function(abspath, normalize=FALSE) {
-  if (!is.character(abspath)) stop("Path not a character string")
-  if (nchar(abspath) < 1 || is.na(abspath)) stop("Path is missing or of zero length")
+#'   \dontrun{
+#'   splitFilePath("dir1/subdir1/mplus1.inp")
+#'   }
+splitFilePath <- function(filepath, normalize=FALSE) {
+  if (!is.character(filepath)) stop("Path not a character string")
+  if (nchar(filepath) < 1 || is.na(filepath)) stop("Path is missing or of zero length")
   
   #trailing slash screws up file.exists call on Windows: https://bugs.r-project.org/bugzilla/show_bug.cgi?id=14721
-  abspath <- sub("(\\\\|/)?$", "", abspath, perl=TRUE)
+  filepath <- sub("(\\\\|/)?$", "", filepath, perl=TRUE)
   
-  components <- strsplit(abspath, split="[\\/]")[[1]]
+  components <- strsplit(filepath, split="[\\/]")[[1]]
   lcom <- length(components)
   
   stopifnot(lcom > 0)
@@ -715,7 +778,7 @@ splitFilePath <- function(abspath, normalize=FALSE) {
     dirpart <- do.call("file.path", as.list(components))
     
     #if path begins with C:, /, ~/, //, or \\, then treat as absolute
-    if (grepl("^([A-Z]{1}:|~/|/|//|\\\\)+.*$", dirpart, perl=TRUE)) { absolute <- TRUE }
+    if (grepl("^([A-Z]{1}:|~/|/|//|\\\\)+.*$", dirpart, perl=TRUE)) absolute <- TRUE
     
     if (normalize) { #convert to absolute path
       dirpart <- normalizePath(dirpart)
@@ -746,8 +809,8 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
   detectionFinished <- FALSE
   line <- 1
   while(detectionFinished == FALSE) {
-    thisLine <- strsplit(modelSection[line], "\\s+", perl=TRUE)[[1]] #assumes that lines are trimmed of leading/trailing whitespace
-    if (line < length(modelSection)) nextLine <- strsplit(modelSection[line+1], "\\s+", perl=TRUE)[[1]]
+    thisLine <- strsplit(trimws(modelSection[line]), "\\s+", perl=TRUE)[[1]]
+    if (line < length(modelSection)) nextLine <- strsplit(trimws(modelSection[line+1]), "\\s+", perl=TRUE)[[1]]
     else nextLine <- NA_character_
     
     if (sectionType == "model_results") {
@@ -759,7 +822,7 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
       
       #Bayesian (ESTIMATOR=BAYES) 6-column output
       if (identical(thisLine, c("Posterior", "One-Tailed", "95%", "C.I.")) &&
-          identical (nextLine, c("Estimate", "S.D.", "P-Value", "Lower", "2.5%", "Upper", "2.5%")))        
+          identical(nextLine, c("Estimate", "S.D.", "P-Value", "Lower", "2.5%", "Upper", "2.5%")))        
         varNames <- c("param", "est", "posterior_sd", "pval", "lower_2.5ci", "upper_2.5ci")
       
       #Bayesian 6-column output for R-SQUARE (Just has "Variable" on the front)
@@ -769,17 +832,17 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
       
       #Bayesian (ESTIMATOR=BAYES) 7-column output (Mplus v7)
       else if (identical(thisLine, c("Posterior", "One-Tailed", "95%", "C.I.")) &&
-          identical (nextLine, c("Estimate", "S.D.", "P-Value", "Lower", "2.5%", "Upper", "2.5%", "Significance")))        
+          identical(nextLine, c("Estimate", "S.D.", "P-Value", "Lower", "2.5%", "Upper", "2.5%", "Significance")))        
         varNames <- c("param", "est", "posterior_sd", "pval", "lower_2.5ci", "upper_2.5ci", "sig")
       
       #Monte Carlo output (e.g., UG ex12.4)
       else if (identical(thisLine, c("ESTIMATES", "S.", "E.", "M.", "S.", "E.", "95%", "%", "Sig")) &&
-          identical (nextLine, c("Population", "Average", "Std.", "Dev.", "Average", "Cover", "Coeff")))        
+          identical(nextLine, c("Population", "Average", "Std.", "Dev.", "Average", "Cover", "Coeff")))        
         varNames <- c("param", "population", "average", "population_sd", "average_se", "mse", "cover_95", "pct_sig_coef")
       
       #Monte Carlo output for STD results
       else if (identical(thisLine, c("ESTIMATES", "S.", "E.", "%", "Sig")) &&
-          identical (nextLine, c("Average", "Std.", "Dev.", "Average", "Coeff")))        
+          identical(nextLine, c("Average", "Std.", "Dev.", "Average", "Coeff")))        
         varNames <- c("param", "average", "population_sd", "average_se", "pct_sig_coef")
       
       #Multiple imputation output (I think format introduced in v7)
@@ -788,7 +851,7 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
         varNames <- c("param", "est", "se", "est_se", "pval", "rate_missing")
       
       #Multiple imputation output for R2 including rate of missing (introduced in v8, I think)
-      else if (identical(thisLine, c("Observed", "Two-Tailed", "Rate", "of")) &&
+      else if ((identical(thisLine, c("Observed", "Two-Tailed", "Rate", "of")) || identical(thisLine, c("Latent", "Two-Tailed", "Rate", "of"))) &&
           identical(nextLine, c("Variable", "Estimate", "S.E.", "Est./S.E.", "P-Value", "Missing")))
         varNames <- c("param", "est", "se", "est_se", "pval", "rate_missing")
       
@@ -802,6 +865,11 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
           identical(nextLine, c("Estimate", "S.E.", "/", "S.E.", "P-Value")))
         varNames <- c("param", "est", "se", "est_se", "pval")
       
+      # Odds ratios in latent class models
+      else if (identical(thisLine, c("95%", "C.I.")) &&
+               identical(nextLine, c("Estimate", "S.E.", "Lower", "2.5%", "Upper", "2.5%")))
+        varNames <- c("param", "est", "se", "lower_2.5ci", "upper_2.5ci")
+      
       #Five-column output for R-Square that applies to most unstandardized and standardized sections in Mplus 5 and later
       else if ((identical(thisLine, c("Observed", "Two-Tailed")) || identical(thisLine, c("Latent", "Two-Tailed"))) &&
           identical(nextLine, c("Variable", "Estimate", "S.E.", "Est./S.E.", "P-Value")))
@@ -879,16 +947,14 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
       else if (identical(thisLine, c("StdYX", "StdY", "Std")) && identical (nextLine, c("Estimate", "Estimate", "Estimate")))
         varNames <- c("param", "stdyx", "stdy", "std")
       
-    }
-    else if (sectionType == "mod_indices") {
+    } else if (sectionType == "mod_indices") {
       nhl <- 1 #default for mod indices
       if (identical(thisLine, c("M.I.", "E.P.C.", "Std", "E.P.C.", "StdYX", "E.P.C."))) {
         varNames <- c("modV1", "operator", "modV2", "MI", "EPC", "Std_EPC", "StdYX_EPC")
       } else if (identical(thisLine, c("M.I.", "E.P.C."))) {
         varNames <- c("modV1", "operator", "modV2", "MI", "EPC") 
       }
-    }
-    else if (sectionType == "confidence_intervals") {
+    } else if (sectionType == "confidence_intervals") {
       nhl <- 1 #default for CIs
       if (identical(thisLine, c("Lower",".5%","Lower","2.5%","Lower","5%",
               "Estimate","Upper","5%","Upper","2.5%","Upper",".5%" ))) {
@@ -897,8 +963,7 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
               "Estimate","Upper","2.5%","Upper",".5%" ))) {
         varNames <- c("param", "low.5", "low2.5", "est", "up2.5", "up.5")
       }
-    }
-    else if (sectionType == "auxe") { #currently unused
+    } else if (sectionType == "auxe") { #currently unused
       nhl <- 1
       if (identical(thisLine, c("Mean", "S.E.", "Mean", "S.E."))) {
         varNames <- c("Mean", "SE", "Mean", "SE")
@@ -1404,3 +1469,17 @@ detectMplus <- function() {
 
   return(mplus)  
 }
+
+# copy elements of append into target. note that data.frames inherit list,
+# so could be wonky if append is a data.frame (shouldn't happen here)
+appendListElements <- function(target, append) {
+  if (!is.list(target)) stop("target is not a list.")
+  if (!is.list(append)) stop("append is not a list.")
+  
+  for (elementName in names(append)) {
+    if (!is.null(target[[elementName]])) warning("Element is already present in target list: ", elementName)
+    target[[elementName]] <- append[[elementName]]
+  }
+  
+  return(target)
+}
